//(de)buff system written by Levis

#event_handler("procRemoveBuffToCharacter","procRemoveBuffToCharacter");

#define DEBUG_BUFFS			3	//(int) on 0 no messages are given. on 1 only error, on 2 simple info is given, on 3 all info is given.

void AddBuffToCharacter(ref rChar, string description, string type, string arg1, string arg2, int arg3, int arg4, float arg5, float arg6, string expire, string exp_arg1, int exp_arg2)
{
	// This function will add a (de)buff to a character. The description will be shown in the character interface to communicate the buff to the player.
	// There are several types of (de)buffs which can be set. Behind the type it will say the arguments which are needed.
	// type = "skill" , if the type is skill a charmod will be set for the character to change this skill. arg1 = skillname, arg2 = description, arg3 = offset
	// type = "attribute" , if the type is set to attribute it will set an attribute for this character. arg1 = attributename, arg2 = value
	// type = "counter" , if the type is set to counter it will add a value to a attribute or set the attribute. arg1 = attributename, arg5 = value
	// type = "hp" , if the type is set to HP it will change the maxhp with the value given. arg1 = ID, arg2 = description, arg3 = changevalue
	// 
	// The expire type determines when the (de)buff will stop working. The expire type can have arguments aswell.
	// expire = "days" , if the expire is set to days it will remove the effect after X amount of days. exp_arg2 = amount of days
	// expire = "exitfromlocation", if exitfromlocation is set it will remove the effect once the player leaves set location. exp_arg1 = location
	// expire = "permanent", (default) if the expire it set to permenent it wont be removed unless it's done manually.
	
	//Check for error
	if(type == "")
	{
		if(DEBUG_BUFFS>0) Trace("BUFFS: Called addbuf but no type is set");
		return;
	}
	
	//Set some variables
	string questname;
	
	//First we need to determine the ID of the buff so we can store it.
	string baseID = "buff_";
	int buffnum = 0;
	string buffID = baseID+buffnum;
	while(CheckAttribute(rChar,"buffs.single."+buffID))
	{
		buffnum++;
		buffID = baseID+buffnum;
	}
	if(DEBUG_BUFFS>1) Trace("BUFFS: Add a new buff with BuffID = "+buffID+" and type = "+type+" and expire = "+expire+" for character "+rChar.id);
	
	//Now we need to determine the values of the buff so this can be set to the character. We do this differently for different types.
	switch(type)
	{
		case "skill":
			//Set the data in the buff
			rChar.buffs.single.(buffID).type = type;
			rChar.buffs.single.(buffID).desc = description;
			rChar.buffs.single.(buffID).args.skill = arg1;
			rChar.buffs.single.(buffID).args.modid = buffID+"_charmod";
			if(DEBUG_BUFFS>2) Trace("BUFFS: Set buff attributes for "+type);
			//Apply the buff
			SetSkillCharMod(rChar, arg1, arg3, rChar.buffs.single.(buffID).args.modid, arg2);
		break;
		
		case "attribute":
			//Set the data in the buff
			rChar.buffs.single.(buffID).type = type;
			rChar.buffs.single.(buffID).desc = description;
			rChar.buffs.single.(buffID).args.attribute = arg1;
			if(DEBUG_BUFFS>2) Trace("BUFFS: Set buff attributes for "+type);
			//Apply the buff
			rChar.(arg1) = arg2;
		break;
		
		case "counter":
			//Set the data in the buff
			rChar.buffs.single.(buffID).type = type;
			rChar.buffs.single.(buffID).desc = description;
			rChar.buffs.single.(buffID).args.attribute = arg1;
			rChar.buffs.single.(buffID).args.value = arg5;
			if(DEBUG_BUFFS>2) Trace("BUFFS: Set buff attributes for "+type);
			//Apply the buff
			if(!CheckAttribute(rChar,arg1)) rChar.(arg1) = arg5;
			else rChar.(arg1) = stf(rChar.(arg1)) + arg5;
		break;
		
		case "hp":
			//Set the data in the buff
			rChar.buffs.single.(buffID).type = type;
			rChar.buffs.single.(buffID).desc = description;
			rChar.buffs.single.(buffID).args.id = arg1;
			if(DEBUG_BUFFS>2) Trace("BUFFS: Set buff attributes for "+type);
			//Apply the buff
			ChangeHPBonus(rChar, arg3, arg1, arg2);
		break;
		
		// default:
		if(DEBUG_BUFFS>0) Trace("BUFFS: type "+type+" is not a valid type for buffs.");
		return;
	}
	
	//If there is an active group we should add it to this
	if(CheckAttribute(rChar,"buffs.activegroup"))
	{
		string groupid = rChar.buffs.activegroup;
		if(DEBUG_BUFFS>1) Trace("BUFFS: Add buff "+buffID+" to group "+groupid);
		rChar.buffs.single.(buffID).group = groupid;
		//Increase the count by 1
		rChar.buffs.group.(groupid) = sti(rChar.buffs.group.(groupid)) + 1;
	}
	
	//Now set the expiration effect
	//We use pchar here because only pchar is checked for quests. So the quest will always be set to pchar. We add the character who needs to be used to the quest so this can be used during the remove function again.
	ref pchar = GetMainCharacter();
	switch(expire)
	{
		case "days":
			//Add data to the interface
			rChar.buffs.single.(buffID).expire = "for "+exp_arg2+" days"; //TODO: translate
			//Set the questcase
			questname = buffID+"_expiration_quest_"+rChar.index; //we want to make sure it's unique
			DeleteAttribute(pchar,"quest."+questname); //remove any previous quest by this
			pchar.quest.(questname).buffID = buffID;
			pchar.quest.(questname).character_idx = rChar.index;
			pchar.quest.(questname).win_condition.l1 = "Timer";
			pchar.quest.(questname).win_condition.l1.date.day = GetAddingDataDay(0, 0, exp_arg2);
			pchar.quest.(questname).win_condition.l1.date.month = GetAddingDataMonth(0, 0, exp_arg2);
			pchar.quest.(questname).win_condition.l1.date.year = GetAddingDataYear(0, 0, exp_arg2);
			pchar.quest.(questname).win_condition = "remove_buff_from_character";
			if(DEBUG_BUFFS>2) Trace("BUFFS: Set expiration "+expire+" with questname = "+questname);
		break;
		
		case "exitfromlocation":
			//Add data to the interface
			rChar.buffs.single.(buffID).expire = "till next area"; //TODO: translate
			//Set the questcase
			questname = buffID+"_expiration_quest_"+rChar.index; //we want to make sure it's unique
			DeleteAttribute(pchar,"quest."+questname); //remove any previous quest by this name
			pchar.quest.(questname).buffID = buffID;
			pchar.quest.(questname).character_idx = rChar.index;
			pchar.quest.(questname).win_condition.l1 = "ExitFromLocation";
			pchar.quest.(questname).win_condition.l1.location = exp_arg1;
			pchar.quest.(questname).win_condition = "remove_buff_from_character";
			if(DEBUG_BUFFS>2) Trace("BUFFS: Set expiration "+expire+" with questname = "+questname);
		break;
		
		// default:
		if(DEBUG_BUFFS>0) Trace("BUFFS: expire "+expire+" is not a valid expire type for buffs.");
		RemoveBuffToCharacter(rChar, buffID);
		return;
	}
}

void RemoveBuffToCharacter(ref rChar, string buffID)
{
	// This function will remove the buff again. It's called by the questcase or other methodes to remove the buffs.
	if(DEBUG_BUFFS>1) Trace("BUFFS: Remove a buff with BuffID = "+buffID+" for character "+rChar.id);
	
	//Check if the buff still exists
	if(!CheckAttribute(rChar,"buffs.single."+buffID))
	{
		if(DEBUG_BUFFS>0) Trace("BUFFS: Trying to remove buff "+buffID+" for character "+rChar.id+" but buff doesn't exist");
		return;
	}
	
	//Get the type from the buff
	string type = rChar.buffs.single.(buffID).type;
	aref args; makearef(args,rChar.buffs.single.(buffID).args);
	
	//Remove the buff effect
	switch(type)
	{
		case "skill":
			if(DEBUG_BUFFS>2) Trace("BUFFS: Remove buff attributes for "+type);
			RemoveSkillCharMod(rChar, args.skill, args.modid)
		break;
		
		case "attribute":
			if(DEBUG_BUFFS>2) Trace("BUFFS: Remove buff attributes for "+type);
			DeleteAttribute(rChar,args.attribute);
		break;
		
		case "counter":
			if(DEBUG_BUFFS>2) Trace("BUFFS: Remove buff attributes for "+type);
			string attr = args.attribute;
			rChar.(attr) = stf(rChar.(attr)) - stf(args.value);
		break;
		
		case "hp":
			if(DEBUG_BUFFS>2) Trace("BUFFS: Remove buff attributes for "+type);
			RemoveHPBonus(rChar,args.id);
		break;
	}
	
	//If the buff has a group we should remove it from there.
	if(CheckAttribute(rChar,"buffs.single."+buffID+".group"))
	{
		string groupid = rChar.buffs.single.(buffID).group;
		if(DEBUG_BUFFS>1) Trace("BUFFS: Remove buff "+buffID+" to group "+groupid);
		//Increase the count by 1
		rChar.buffs.group.(groupid) = sti(rChar.buffs.group.(groupid)) - 1;
		//Check if the group needs to be removed
		if(sti(rChar.buffs.group.(groupid)) < 1)
		{
			if(DEBUG_BUFFS>1) Trace("BUFFS: Remove group "+groupid+" because count is lower then 1");
			DeleteAttribute(rChar,"buffs.group."+groupid);
		}
	}
	
	//Remove the buff from the character
	DeleteAttribute(rChar,"buffs.single."+buffID);
}

void procRemoveBuffToCharacter()
{
	//This function is used to call the remove function with an event
	aref rChar = GetEventData();
	string buffID = GetEventData();
	RemoveBuffToCharacter(rChar, buffID);
}

void StartGroupAddBuffToCharacter(ref rChar, string buffgroupID, string description)
{
	//This will group the buffs added to this character until the stop command is called. These buffs will be shown together in the interface with this description.
	
	//We check if it's a prededfined group or not
	if(buffgroupID == "")
	{
		//First we need to determine the ID of the buffgroup so we can store it.
		string baseID = "buffgroup_";
		int buffnum = 0;
		buffgroupID = baseID+buffnum;
		while(CheckAttribute(rChar,"buffs.group."+buffgroupID))
		{
			buffnum++;
			buffgroupID = baseID+buffnum;
		}
	}
	if(DEBUG_BUFFS>1) Trace("BUFFS: Start a buffgroup with BuffgroupID = "+buffgroupID+" for character "+rChar.id);
	
	//Now create the group
	rChar.buffs.group.(buffgroupID) = 0; //In here we will store how many buffs are added to this group
	rChar.buffs.group.(buffgroupID).desc = description;
	
	//Set this as active group so we know to add to this group now
	rChar.buffs.activegroup = buffgroupID;
}

void StopGroupAddBuffToCharacter(ref rChar)
{
	//This function makes the system stop adding the buffs to this specific group and instead adds them as single buffs again. Make sure to always call this after a startgroup command is called.
	
	if(!CheckAttribute(rChar,"buffs.activegroup"))
	{
		if(DEBUG_BUFFS>0) Trace("BUFFS: Trying to stop adding buffs to activegroup for character "+rChar.id+" but there is no active bufgroup");
		return;
	}
	
	DeleteAttribute(rChar,"buffs.activegroup");
}